# $Id: Node.pm,v 1.4 2002/01/30 22:48:33 joern Exp $

#-----------------------------------------------------------------------
# Copyright (C) 2001-2002 Jrn Reder <joern@zyn.de> All Rights Reserved
# 
# This program is part of Video::DVDRip, which is free software; you can
# redistribute it and/or modify it under the same terms as Perl itself.
#-----------------------------------------------------------------------

package Video::DVDRip::Cluster::Node;

use base Video::DVDRip::Base;

use Carp;
use strict;

use FileHandle;
use Data::Dumper;

use Net::Ping;

sub filename			{ shift->{filename}			}
sub name			{ shift->{name}				}
sub data_base_dir		{ shift->{data_base_dir}		}
sub is_master			{ shift->{is_master}			}
sub data_is_local		{ shift->{data_is_local}		}
sub username			{ shift->{username}			}
sub speed			{ shift->{speed}			}

sub progress_frames		{ shift->{progress_frames}		}
sub progress_frames_cnt		{ shift->{progress_frames_cnt}		}
sub progress_merge		{ shift->{progress_merge}		}
sub progress_start_time		{ shift->{progress_start_time}		}
sub assigned_job		{ shift->{assigned_job} 		}
sub assigned_chunk		{ shift->{assigned_chunk}		}

sub set_filename		{ shift->{filename}		= $_[1] }
sub set_name			{ shift->{name}			= $_[1] }
sub set_data_base_dir		{ shift->{data_base_dir}	= $_[1] }
sub set_is_master		{ shift->{is_master}		= $_[1] }
sub set_data_is_local		{ shift->{data_is_local}	= $_[1] }
sub set_username		{ shift->{username}		= $_[1] }
sub set_speed			{ shift->{speed}		= $_[1] }

sub set_progress_frames		{ shift->{progress_frames}	= $_[1] }
sub set_progress_frames_cnt	{ shift->{progress_frames_cnt}	= $_[1] }
sub set_progress_merge		{ shift->{progress_merge}	= $_[1] }
sub set_progress_start_time	{ shift->{progress_start_time}	= $_[1] }
sub set_assigned_job		{ shift->{assigned_job} 	= $_[1] }
sub set_assigned_chunk		{ shift->{assigned_chunk}	= $_[1] }

sub alive			{ shift->{alive}			}

sub set_alive {
	my $self = shift;
	my ($alive) = @_;
	
	my $was_alive = $self->alive;

	$self->{alive} = $alive;
	
	if ( not $alive ) {
		$self->stop if $was_alive;
		$self->force_state ("offline");
		$self->save;

	} elsif ( $self->state eq "offline" or
		  $self->state eq "unknown" ) {
		$self->force_state ("idle");
		$self->save;
	}

	1;
}

sub progress_fps {
	my $self = shift;
	return sprintf (
		"%3.1f", $self->progress_frames /
			 ((time - $self->progress_start_time)||1)
	);
}

sub project_name {
	my $self = shift;
	my $job = $self->assigned_job;
	return "" if not $job;
	return $job->project->label;
}

my %KNOWN_STATES = (
	'unknown'          => { ''	    => 1, stopped => 1,    },
	'idle'             => { unknown     => 1, offline => 1,
			   	transcoding => 1, merging => 1,
			   	stopped     => 1,
				'audio processing' => 1		   },
	'transcoding'      => { idle	    => 1		   },
	'merging'          => { idle	    => 1		   },
	'audio processing' => { idle	    => 1		   },
	'offline'          => { unknown     => 1, idle    => 1,
			   	transcoding => 1, merging => 1     },
	'stopped'	   => { idle	    => 1		   },
);

sub state		{ shift->{state}			}

sub set_state {
	my $self = shift;
	my ($state) = @_;
	
	croak "Unknown state '$state'"
		if not defined $KNOWN_STATES{$state};

	my $old_state = $self->state;
	
	croak "Illegal state change from '$old_state' to '$state'"
		if $state ne $old_state and
		   not $KNOWN_STATES{$state}->{$old_state};

	$self->{state} = $state;
	
	return $state;
}

sub force_state {
	my $self = shift;
	my ($state) = @_;
	
	croak "Unknown state '$state'"
		if not defined $KNOWN_STATES{$state};
	
	$self->{state} = $state;
	
	return $state;
}

sub new {
	my $class = shift;
	my %par = @_;
	my  ($name, $data_base_dir, $is_master, $username) =
	@par{'name','data_base_dir','is_master','username'};

	my $self = {
		name 		=> $name,
		data_base_dir 	=> $data_base_dir,
		is_master 	=> $is_master,
		username	=> $username,
		alive           => 0,
	};
	
	bless $self, $class;
	
	$self->set_state ( "unknown" );
	
	return $self;
}

sub new_from_file {
	my $class = shift;
	my %par = @_;
	my  ($filename) =
	@par{'filename'};
	
	confess "missing filename" if not $filename;
	
	my $self = bless {
		filename => $filename,
	}, $class;
	
	$self->load;
	
	$self->set_filename ($filename);

	return $self;
}

sub load {
	my $self = shift; $self->trace_in;
	
	my $filename = $self->filename;
	croak "no filename set" if not $filename;
	croak "can't read $filename" if not -r $filename;
	
	my $fh = FileHandle->new;
	open ($fh, $filename) or croak "can't read $filename";
	my $data_blob = join ('', <$fh>);
	close $fh;

	my $data;
	$data = eval($data_blob);
	croak "can't load $filename. Perl error: $@" if $@;

	%{$self} = %{$data};

	1;
}

sub save {
	my $self = shift; $self->trace_in;
	
	my $filename = $self->filename;
	confess "not filename set" if not $filename;
	
	my $assigned_job = $self->assigned_job;
	$self->set_assigned_job( undef );
	
	my $dd = Data::Dumper->new ( [$self], ['data'] );
	$dd->Indent(1);
	my $data = $dd->Dump;

	$self->set_assigned_job ( $assigned_job );

	my $fh = FileHandle->new;

	open ($fh, "> $filename") or confess "can't write $filename";
	print $fh $data;
	close $fh;
	
	1;
}

sub get_popen_code {
	my $self = shift;
	my %par = @_;
	my ($command) = @par{'command'};
	
	return $command if $self->is_master;

	my $username = $self->username;
	my $name     = $self->name;

	$command =~ s/"/\\"/g;
	$command = qq{ssh -C -P $username\@$name "$command"};
	
	return $command;
}

sub reset {
	my $self = shift;
	
	$self->force_state ('unknown') if $self->state ne 'stopped';

	1;
}

sub progress {
	my $self = shift;
	
	my $state = $self->state;

	if ( $state eq 'transcoding' ) {
		# transcoding progress
		my $frames     = $self->progress_frames;
		my $frames_cnt = $self->progress_frames_cnt || 1;
		my $fps        = $self->progress_fps;
		my $title      = $self->assigned_job->project->title;
		return sprintf (
			"#%d, ".
			"%2.2f\%, %2.1f fps",
			$self->assigned_chunk,
			$frames / $frames_cnt * 100,
			$fps
		);

	} elsif ( $state eq 'merging' ) {
		# merging progress
		return sprintf (
			"Chunk %d/%d",
			$self->progress_merge,
			$self->assigned_job->project->title->chunk_cnt
		);

	} elsif ( $state eq 'audio processing' ) {
		# audio processing progress
		my $frames     = $self->progress_frames;
		my $frames_cnt = $self->progress_frames_cnt || 1;
		my $fps        = $self->progress_fps;
		my $title      = $self->assigned_job->project->title;
		return sprintf (
			"%2.2f\%, %2.1f fps",
			$frames / $frames_cnt * 100,
			$fps
		);
	}
	
	return "";
}

sub stop {
	my $self = shift;
	
	my $job = $self->assigned_job;

	if ( $job ) {
		$job->set_stopping_in_progress(1);
		$job->abort;

	} else {
		$self->log ("Node '".$self->name."' stopped");
		$self->set_state ('idle');
		$self->set_state ('stopped');
		$self->save;
	}
	
	1;
}

sub start {
	my $self = shift;
	
	croak "Can't start a non stopped node"
		if $self->state ne 'stopped';

	$self->log ("Node '".$self->name."' started");

	$self->set_state ('unknown');
	$self->save;

	Video::DVDRip::Cluster::Master->get_master->job_control;
	
	1;
}

1;
